//
//  ViewController.swift
//  BLENotifyExamplePeripheral
//
//  Created by Shuichi Tsutsumi on 2014/12/12.
//  Copyright (c) 2014 Shuichi Tsutsumi. All rights reserved.
//

import UIKit
import CoreBluetooth

class ViewController: UIViewController, CBPeripheralManagerDelegate {

    @IBOutlet var advertiseBtn: UIButton!
    @IBOutlet var valueLabel: UILabel!
    
    private var peripheralManager: CBPeripheralManager!
    private let serviceUUID = CBUUID(string: "0000")
    private var characteristic: CBMutableCharacteristic!

    override func viewDidLoad() {
        super.viewDidLoad()
        
        // ペリフェラルマネージャ初期化
        peripheralManager = CBPeripheralManager(delegate: self, queue: nil, options: nil)
        
        valueLabel.text = nil;
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
    
    // =========================================================================
    // MARK: Private
    
    private func publishservice () {
        
        // サービスを作成
        let service = CBMutableService(type: serviceUUID, primary: true)
        
        // キャラクタリスティックを作成
        let characteristicUUID = CBUUID(string: "0001")
        let properties: CBCharacteristicProperties = [.Notify, .Read, .Write]
        let permissions: CBAttributePermissions = [.Readable, .Writeable]
        characteristic = CBMutableCharacteristic(
            type: characteristicUUID,
            properties: properties,
            value: nil,
            permissions: permissions)
        
        // キャラクタリスティックをサービスにセット
        service.characteristics = [characteristic]
        
        // サービスを Peripheral Manager にセット
        peripheralManager.addService(service)
        
        // 値をセット
        let value = UInt8(arc4random() & 0xFF)
        let data = NSData(bytes: [value], length: 1)
        characteristic.value = data;
    }
    
    private func startAdvertise() {
        
        // アドバタイズメントデータを作成する
        let advertisementData = [
            CBAdvertisementDataLocalNameKey: "Test Device",
            CBAdvertisementDataServiceUUIDsKey: [serviceUUID]
        ]
        
        // アドバタイズ開始
        peripheralManager.startAdvertising(advertisementData)
        
        advertiseBtn.setTitle("STOP ADVERTISING", forState: UIControlState.Normal)
    }
    
    private func stopAdvertise () {
        
        // アドバタイズ停止
        peripheralManager.stopAdvertising()
        
        advertiseBtn.setTitle("START ADVERTISING", forState: UIControlState.Normal)
    }
    
    private func updateValueLabel () {
        valueLabel.text = "Characteristic value: \(characteristic.value)"
    }
    
    // =========================================================================
    // MARK: CBPeripheralManagerDelegate
    
    // ペリフェラルマネージャの状態が変化すると呼ばれる
    func peripheralManagerDidUpdateState(peripheral: CBPeripheralManager) {
        print("state: \(peripheral.state)")
        
        switch peripheral.state {
        case .PoweredOn:
            // サービス登録開始
            publishservice()
        default:
            break
        }
    }
    
    // サービス追加処理が完了すると呼ばれる
    func peripheralManager(peripheral: CBPeripheralManager, didAddService service: CBService, error: NSError?) {
        if let error = error {
            print("サービス追加失敗！ error: \(error)")
            return
        }
        print("サービス追加成功！")
        
        // アドバタイズ開始
        startAdvertise()
    }
    
    // アドバタイズ開始処理が完了すると呼ばれる
    func peripheralManagerDidStartAdvertising(peripheral: CBPeripheralManager, error: NSError?) {
        if let error = error {
            print("アドバタイズ開始失敗！ error: \(error)")
            return
        }
        print("アドバタイズ開始成功！")
    }
    
    // Readリクエスト受信時に呼ばれる
    func peripheralManager(peripheral: CBPeripheralManager, didReceiveReadRequest request: CBATTRequest) {
        print("Readリクエスト受信！ requested service uuid:\(request.characteristic.service.UUID) characteristic uuid:\(request.characteristic.UUID) value:\(request.characteristic.value)")
        
        // プロパティで保持しているキャラクタリスティックへのReadリクエストかどうかを判定
        if request.characteristic.UUID.isEqual(characteristic.UUID)
        {
            // CBMutableCharacteristicのvalueをCBATTRequestのvalueにセット
            request.value = characteristic.value;
            
            // リクエストに応答
            peripheralManager.respondToRequest(request, withResult: .Success)
        }
    }
    
    // Writeリクエスト受信時に呼ばれる
    func peripheralManager(peripheral: CBPeripheralManager, didReceiveWriteRequests requests: [CBATTRequest]) {
        print("\(requests.count) 件のWriteリクエストを受信！")
        
        for request in requests {
            print("Requested value:\(request.value) service uuid:\(request.characteristic.service.UUID) characteristic uuid:\(request.characteristic.UUID)")
            
            if request.characteristic.UUID.isEqual(characteristic.UUID)
            {
                // CBCharacteristicのvalueに、CBATTRequestのvalueをセット
                characteristic.value = request.value;
            }
        }
        // リクエストに応答
        peripheralManager.respondToRequest(requests[0] , withResult: .Success)
    }
    
    // Notify開始リクエスト受信時に呼ばれる
    func peripheralManager(
        peripheral: CBPeripheralManager,
        central: CBCentral,
        didSubscribeToCharacteristic characteristic: CBCharacteristic)
    {
        print("Notify開始リクエストを受信")
        guard let characteristic = characteristic as? CBMutableCharacteristic else {fatalError()}
        print("Notify中のセントラル: \(characteristic.subscribedCentrals)")
    }
    
    // Notify停止リクエスト受信時に呼ばれる
    func peripheralManager(peripheral: CBPeripheralManager, central: CBCentral, didUnsubscribeFromCharacteristic characteristic: CBCharacteristic)
    {
        print("Notify停止リクエストを受信")
        guard let characteristic = characteristic as? CBMutableCharacteristic else {fatalError()}
        print("Notify中のセントラル: \(characteristic.subscribedCentrals)")
    }
    
    // =========================================================================
    // MARK: Actions
    
    @IBAction func advertiseBtnTapped(sender: UIButton) {
        if !peripheralManager.isAdvertising {
            startAdvertise()
        } else {
            stopAdvertise()
        }
    }
    
    @IBAction func updateBtnTapped(sender: UIButton) {
        // 新しい値となるNSDataオブジェクトを生成
        let value = UInt8(arc4random() & 0xFF)
        let data = NSData(bytes: [value] as [UInt8], length: 1)
        
        // 値を更新
        characteristic.value = data;

        let result =  peripheralManager.updateValue(
            data,
            forCharacteristic: characteristic,
            onSubscribedCentrals: nil)
        
        print("result: \(result)")
        
        updateValueLabel()
    }
}

